@@ -1,4 +1,4 @@
-/* $OpenBSD: s3_pkt.c,v 1.53 2014/12/14 15:30:50 jsing Exp $ */
+/* $OpenBSD: s3_pkt.c,v 1.55 2015/07/18 19:41:54 doug Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -117,6 +117,8 @@
 #include <openssl/buffer.h>
 #include <openssl/evp.h>
 
+#include "bytestring.h"
+
 static int do_ssl3_write(SSL *s, int type, const unsigned char *buf,
     unsigned int len, int create_empty_fragment);
 static int ssl3_get_record(SSL *s);
@@ -276,64 +278,56 @@ ssl3_read_n(SSL *s, int n, int max, int extend)
 static int
 ssl3_get_record(SSL *s)
 {
-	int ssl_major, ssl_minor, al;
+	int al;
 	int enc_err, n, i, ret = -1;
 	SSL3_RECORD *rr;
 	SSL_SESSION *sess;
-	unsigned char *p;
 	unsigned char md[EVP_MAX_MD_SIZE];
-	short version;
 	unsigned mac_size, orig_len;
-	size_t extra;
 
 	rr = &(s->s3->rrec);
 	sess = s->session;
 
-	if (s->options & SSL_OP_MICROSOFT_BIG_SSLV3_BUFFER)
-		extra = SSL3_RT_MAX_EXTRA;
-	else
-		extra = 0;
-
-	if (extra && !s->s3->init_extra) {
-		/* An application error: SLS_OP_MICROSOFT_BIG_SSLV3_BUFFER
-		 * set after ssl3_setup_buffers() was done */
-		SSLerr(SSL_F_SSL3_GET_RECORD, ERR_R_INTERNAL_ERROR);
-		return -1;
-	}
-
 again:
 	/* check if we have the header */
 	if ((s->rstate != SSL_ST_READ_BODY) ||
 	    (s->packet_length < SSL3_RT_HEADER_LENGTH)) {
+		CBS header;
+		uint16_t len, ssl_version;
+		uint8_t type;
+
 		n = ssl3_read_n(s, SSL3_RT_HEADER_LENGTH, s->s3->rbuf.len, 0);
 		if (n <= 0)
 			return(n); /* error or non-blocking */
 		s->rstate = SSL_ST_READ_BODY;
 
-		p = s->packet;
+		CBS_init(&header, s->packet, n);
 
 		/* Pull apart the header into the SSL3_RECORD */
-		rr->type= *(p++);
-		ssl_major= *(p++);
-		ssl_minor= *(p++);
-		version = (ssl_major << 8)|ssl_minor;
-		n2s(p, rr->length);
-
-		/* Lets check version */
-		if (!s->first_packet) {
-			if (version != s->version) {
-				SSLerr(SSL_F_SSL3_GET_RECORD,
-				    SSL_R_WRONG_VERSION_NUMBER);
-				if ((s->version & 0xFF00) == (version & 0xFF00) &&
-				    !s->enc_write_ctx && !s->write_hash)
-					/* Send back error using their minor version number :-) */
-					s->version = (unsigned short)version;
-				al = SSL_AD_PROTOCOL_VERSION;
-				goto f_err;
-			}
+		if (!CBS_get_u8(&header, &type) ||
+		    !CBS_get_u16(&header, &ssl_version) ||
+		    !CBS_get_u16(&header, &len)) {
+			SSLerr(SSL_F_SSL3_GET_RECORD,
+			    SSL_R_BAD_PACKET_LENGTH);
+			goto err;
 		}
 
-		if ((version >> 8) != SSL3_VERSION_MAJOR) {
+		rr->type = type;
+		rr->length = len;
+
+		/* Lets check version */
+		if (!s->first_packet && ssl_version != s->version) {
+			SSLerr(SSL_F_SSL3_GET_RECORD,
+			    SSL_R_WRONG_VERSION_NUMBER);
+			if ((s->version & 0xFF00) == (ssl_version & 0xFF00) &&
+			    !s->enc_write_ctx && !s->write_hash)
+				/* Send back error using their minor version number :-) */
+				s->version = ssl_version;
+			al = SSL_AD_PROTOCOL_VERSION;
+			goto f_err;
+		}
+
+		if ((ssl_version >> 8) != SSL3_VERSION_MAJOR) {
 			SSLerr(SSL_F_SSL3_GET_RECORD,
 			    SSL_R_WRONG_VERSION_NUMBER);
 			goto err;
@@ -379,7 +373,7 @@ again:
 	 * rr->length bytes of encrypted compressed stuff. */
 
 	/* check is not needed I believe */
-	if (rr->length > SSL3_RT_MAX_ENCRYPTED_LENGTH + extra) {
+	if (rr->length > SSL3_RT_MAX_ENCRYPTED_LENGTH) {
 		al = SSL_AD_RECORD_OVERFLOW;
 		SSLerr(SSL_F_SSL3_GET_RECORD, SSL_R_ENCRYPTED_LENGTH_TOO_LONG);
 		goto f_err;
@@ -449,7 +443,7 @@ again:
 		    timingsafe_memcmp(md, mac, (size_t)mac_size) != 0)
 			enc_err = -1;
 		if (rr->length >
-		    SSL3_RT_MAX_COMPRESSED_LENGTH + extra + mac_size)
+		    SSL3_RT_MAX_COMPRESSED_LENGTH + mac_size)
 			enc_err = -1;
 	}
 
@@ -468,7 +462,7 @@ again:
 		goto f_err;
 	}
 
-	if (rr->length > SSL3_RT_MAX_PLAIN_LENGTH + extra) {
+	if (rr->length > SSL3_RT_MAX_PLAIN_LENGTH) {
 		al = SSL_AD_RECORD_OVERFLOW;
 		SSLerr(SSL_F_SSL3_GET_RECORD, SSL_R_DATA_LENGTH_TOO_LONG);
 		goto f_err;
